#! /usr/bin/env python

import os
import sys
import subprocess
from pathlib import Path
from setuptools import Extension, find_packages, setup
from Cython.Build import cythonize

try:
    import builtins
except ImportError:
    import __builtin__ as builtins

builtins.__MLSAUCE_SETUP__ = True

DISTNAME = 'mlsauce'
DESCRIPTION = 'Miscellaneous Statistical/Machine Learning tools'
LONG_DESCRIPTION = 'Miscellaneous Statistical/Machine Learning tools'
MAINTAINER = 'T. Moudiki'
MAINTAINER_EMAIL = 'thierry.moudiki@gmail.com'
LICENSE = 'BSD'
VERSION = '0.25.3'

install_requires = [
    "numpy",
    "Cython",
    "jax",
    "jaxlib",
    "joblib",
    "matplotlib",
    "nnetsauce",
    "pandas",
    "requests",
    "scikit-learn",
    "scipy",
    "tqdm"
]

# Ensure Cython and NumPy are installed
try:
    subprocess.run(['uv', 'pip', 'install', 'numpy'], check=True)
    subprocess.run(['uv', 'pip', 'install', 'Cython'], check=True)
except Exception:
    subprocess.run(['pip', 'install', 'numpy'])
    subprocess.run(['pip', 'install', 'Cython'])

import numpy

script_dir = Path(__file__).parent.resolve()

ext_modules = [
    Extension("mlsauce.adaopt._adaoptc",
              [str(script_dir / "mlsauce/adaopt/_adaoptc.pyx")],
              include_dirs=[numpy.get_include()]),

    Extension("mlsauce.booster._boosterc",
              [str(script_dir / "mlsauce/booster/_boosterc.pyx")],
              include_dirs=[numpy.get_include()]),

    Extension("mlsauce.lasso._lassoc",
              [str(script_dir / "mlsauce/lasso/_lassoc.pyx")],
              include_dirs=[numpy.get_include()]),

    Extension("mlsauce.ridge._ridgec",
              [str(script_dir / "mlsauce/ridge/_ridgec.pyx")],
              include_dirs=[numpy.get_include()]),

    Extension("mlsauce.stump._stumpc",
              [str(script_dir / "mlsauce/stump/_stumpc.pyx")],
              include_dirs=[numpy.get_include()])
]


if __name__ == "__main__":
    setup(
        name=DISTNAME,
        maintainer=MAINTAINER,
        maintainer_email=MAINTAINER_EMAIL,
        description=DESCRIPTION,
        license=LICENSE,
        version=VERSION,
        long_description=LONG_DESCRIPTION,
        classifiers=[
            'Development Status :: 3 - Alpha',
            'Intended Audience :: Developers',
            'Natural Language :: English',
            "License :: OSI Approved :: BSD License",
            'Programming Language :: Python :: 3',
            'Programming Language :: Python :: 3.5',
            'Programming Language :: Python :: 3.6',
            'Programming Language :: Python :: 3.7',
            'Programming Language :: Python :: 3.8',
        ],
        platforms=["linux", "macosx", "windows"],
        python_requires=">=3.5",
        install_requires=install_requires,
        setup_requires=["numpy", "Cython"],
        packages=find_packages(),
        ext_modules=cythonize(ext_modules, annotate=True),
        package_data={'': ['*.pxd']}
    )

    dir_path = os.path.dirname(os.path.realpath(__file__))
    folders = ['adaopt', 'booster', 'lasso', 'ridge', 'stump']
    for folder in folders: 
        filename = os.path.join(dir_path, "mlsauce", folder, 'setup2.py')                
        try: 
            subprocess.run(['python3', filename, 'build_ext', '--inplace'])  
        except Exception as e:
            print(f"Error running setup for {folder}: {e}")
            subprocess.run(['python', filename, 'build_ext', '--inplace'])  



import numpy as np
import pandas as pd


def create_histogram_with_bin_values(x):
    """
    Computes a histogram for the input data and assigns a value to each bin
    reflecting the ordering of the input.
    
    Args:
    x (list or np.array): Input data.
    num_bins (int): Number of bins for the histogram.
    
    Returns:
    bin_edges (np.array): The edges of the bins.
    bin_value_dict (dict): A dictionary where keys are the bin ranges (tuples) and values reflect the ordering.
    """
    # Compute the histogram
    _, bin_edges = np.histogram(x, bins="auto")

    bin_edges = np.concatenate([[-1e10], bin_edges, [1e10]]).ravel()        
        
    return {i: (bin_edges[i], bin_edges[i + 1]) for i in range(len(bin_edges) - 1)}


def assign_values_to_input(x, bin_value_dict):
    """
    Assigns values to a new input based on the provided bin ranges and values.

    Args:
    x (list or np.array): New input data to assign values to.
    bin_value_dict (dict): Dictionary where keys are bin ranges (tuples) and values are the assigned values.

    Returns:
    assigned_values (list): List of assigned values for the new input data.
    """

    if np.issubdtype(x.dtype, np.int64) or np.issubdtype(x.dtype, np.object_):

        return x

    assigned_values = []

    for value in x:
        assigned = None
        # Find the appropriate bin for each value
        for i, elt in enumerate(bin_value_dict.items()):
            if elt[1][0] < value <= elt[1][1]:
                assigned = float(i)
                break

        assigned_values.append(assigned)

    return np.asarray(assigned_values).ravel()


def get_histo_features(X, bin_value_dict=None):
    """
    Computes histogram features for the input data.
    
    Args:
    X {np.array or pd.DataFrame}: Input data.
    
    Returns:
    X_hist {np.array or pd.DataFrame}: Input data with histogram features.
    """

    if bin_value_dict is None: # training set case        

        if isinstance(X, pd.DataFrame):
            colnames = X.columns
            X = X.values
            X_hist = pd.DataFrame(np.zeros(X.shape), 
                                        columns=colnames)
            for i, col in enumerate(colnames):
                bin_value_dict = create_histogram_with_bin_values(X[:, i])
                X_hist[col] = assign_values_to_input(X[:, i], bin_value_dict)
        else: 
            X_hist = np.zeros(X.shape)        
            for i in range(X.shape[1]):
                bin_value_dict = create_histogram_with_bin_values(X[:, i])
                X_hist[:, i] = assign_values_to_input(X[:, i], bin_value_dict)            
            
        return X_hist, bin_value_dict

    else: # test set case
            
            if isinstance(X, pd.DataFrame):
                colnames = X.columns
                X = X.values
                X_hist = pd.DataFrame(np.zeros(X.shape), 
                                            columns=colnames)
                for i, col in enumerate(colnames):
                    X_hist[col] = assign_values_to_input(X[:, i], bin_value_dict)
            else: 
                X_hist = np.zeros(X.shape)        
                for i in range(X.shape[1]):
                    X_hist[:, i] = assign_values_to_input(X[:, i], bin_value_dict)            
    
            return X_hist


try: 
    from .lazyboosterclassif import LazyBoostingClassifier
    from .lazyboosterregression import LazyBoostingRegressor
    from .lazyboostermts import LazyBoostingMTS
except ImportError as e:
    print(f"Could not import lazybooster modules: {e}")

__all__ = ["LazyBoostingClassifier", "LazyBoostingRegressor", "LazyBoostingMTS"]
